using System;
using System.Data;
using System.Data.Common;
using System.Data.Odbc;
using System.Configuration;

namespace OnlinePhotoShop
{
	/// <summary>
	/// From CustomerDB inherited class that creates data adapter for Odbc data sources.
	/// </summary>
	public class OdbcCustomerDB : CustomerDB
	{
		/// <summary>
		/// Returns the database connection string read from the application setting DBConnection.
		/// </summary>
		protected string ConnectionString
		{
			get 
			{
				return ConfigurationSettings.AppSettings["DBConnection"];
			}
		}

		/// <summary>
		/// Returns the table where the customer records are stored read from the application setting CustomerTable.
		/// </summary>
		protected string TableName
		{
			get
			{
				return ConfigurationSettings.AppSettings["CustomerTable"];
			}
		}

		/// <summary>
		/// Creates a new data adapter for the customer database.
		/// </summary>
		/// <param name="selectCondition">Condition attached to the SELECT command.</param>
		/// <returns>DbDataAdapter to the customer database.</returns>
		public override DbDataAdapter CreateDataAdapter(string selectCondition)
		{
			OdbcConnection conn = new OdbcConnection(ConnectionString);
			OdbcDataAdapter dataAdapter = new OdbcDataAdapter();

			// Select Command
			OdbcCommand cmdSelect = new OdbcCommand();
			dataAdapter.SelectCommand = cmdSelect;
			cmdSelect.CommandText = "SELECT * FROM " + this.TableName + " " + selectCondition;
			cmdSelect.CommandType = CommandType.Text;
			cmdSelect.Connection = conn;

			//Update Command
			OdbcCommand cmdUpdate = new OdbcCommand();
			dataAdapter.UpdateCommand = cmdUpdate;
			cmdUpdate.CommandText = "UPDATE [Customers$] SET " +
				"Name = ?,Address1 = ?,Address2 = ?,City = ?,State = ?,Zip = ?," +
				"CCType = ?,CCNumber = ?,CCExpMonth = ?,CCExpYear = ? " +
				"WHERE Email = ?";
			cmdUpdate.CommandType = CommandType.Text;
			cmdUpdate.Connection = conn;
			cmdUpdate.Parameters.Add(new OdbcParameter("Name", OdbcType.VarChar, 50, "Name"));
			cmdUpdate.Parameters.Add(new OdbcParameter("Address1", OdbcType.VarChar, 50, "Address1"));
			cmdUpdate.Parameters.Add(new OdbcParameter("Address2", OdbcType.VarChar, 50, "Address2"));
			cmdUpdate.Parameters.Add(new OdbcParameter("City", OdbcType.VarChar, 50, "City"));
			cmdUpdate.Parameters.Add(new OdbcParameter("State", OdbcType.VarChar, 2, "State"));
			cmdUpdate.Parameters.Add(new OdbcParameter("Zip", OdbcType.Int, 5, "Zip"));
			cmdUpdate.Parameters.Add(new OdbcParameter("CCType", OdbcType.VarChar, 10, "CCType"));
			cmdUpdate.Parameters.Add(new OdbcParameter("CCNumber", OdbcType.VarChar, 16, "CCNumber"));
			cmdUpdate.Parameters.Add(new OdbcParameter("CCExpMonth", OdbcType.Int, 2, "CCExpMonth"));
			cmdUpdate.Parameters.Add(new OdbcParameter("CCExpYear", OdbcType.Int, 4, "CCExpYear"));
			cmdUpdate.Parameters.Add(new OdbcParameter("Email", OdbcType.VarChar, 50, "Email"));

			//Insert Command
			OdbcCommand cmdInsert = new OdbcCommand();
			dataAdapter.InsertCommand = cmdInsert;
			cmdInsert.CommandText = "INSERT INTO [Customers$] (Email, Password) VALUES (?,?)";
			cmdInsert.CommandType = CommandType.Text;
			cmdInsert.Connection = conn;
			cmdInsert.Parameters.Add(new OdbcParameter("Email",OdbcType.VarChar,50,"Email"));
			cmdInsert.Parameters.Add(new OdbcParameter("Password",OdbcType.VarChar,50,"Password"));

			return dataAdapter;
		}
	}
}
