using System;

namespace OnlinePhotoShop
{
	/// <summary>
	/// Formatted version of ShoppingItem for displaying it in the shopping cart table.
	/// </summary>
	/// <requirement>F:product_shopping_cart</requirement>
	public class ShoppingItemView
	{
		/// <summary>
		/// Private member to store the hashtable key of the item.
		/// </summary>
		int mKey;
		/// <summary>
		/// Private member to store the hashtable key of the item.
		/// </summary>
		string mProduct;
		/// <summary>
		/// Private member to store the product name.
		/// </summary>
		string mOption;
		/// <summary>
		/// Private member to store the option name.
		/// </summary>
		int mQuantity;
		/// <summary>
		/// Private member to store the quantity of the ordered item.
		/// </summary>
		string mImage;
		/// <summary>
		/// Private member to store the image path.
		/// </summary>
		double mPrice;

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="file">Path of the product catalog file.</param>
		/// <param name="key">Shopping cart hashtable key of the item.</param>
		/// <param name="item">Shopping item that will be formatted for output.</param>
		public ShoppingItemView(string file, int key, ShoppingItem item)
		{
			mKey = key;
			mImage = item.ClientPath;
			mQuantity = item.Quantity;

			// get the product name
			ProductParser parser = new ProductParser(file);
			mProduct =
				(string) parser.ProductDetails(item.Product)["name"];

			// get the option name and item price
			System.Collections.Hashtable details =
				parser.OptionDetails(item.Product, item.Option);
			mOption = (string) details["name"];
			double price =
				System.Convert.ToDouble((string) details["price"]);
			mPrice = (price * item.Quantity);
		}

		/// <summary>
		/// Key in shopping cart hashtable.
		/// </summary>
		public string Key
		{
			get
			{
				return mKey.ToString();
			}
		}

		/// <summary>
		/// Product name.
		/// </summary>
		public string Product
		{
			get 
			{
				return mProduct;
			}
		}

		/// <summary>
		/// Option name.
		/// </summary>
		public string Option
		{
			get 
			{
				return mOption;
			}
		}

		/// <summary>
		/// Quantity.
		/// </summary>
		public int Quantity
		{
			get 
			{
				return mQuantity;
			}
		}

		/// <summary>
		/// Path of image file.
		/// </summary>
		public string Image
		{
			get 
			{
				return mImage;
			}
		}

		/// <summary>
		/// Formatted price for this item.
		/// </summary>
		public string Price
		{
			get
			{
				return mPrice.ToString("C");
			}
		}

		/// <summary>
		/// Unformatted price for this item.
		/// </summary>
		public double Subtotal
		{
			get
			{
				return mPrice;
			}
		}
	}
}
