using System;
using System.Collections;
using System.Data;
using System.Data.Common;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Web.Security;
using System.Configuration;
using System.Xml;
using System.IO;

namespace OnlinePhotoShop.checkout
{
	/// <summary>
	/// Checkout form collecting shipping and payment information.
	/// </summary>
	/// <requirement>F:order_checkout</requirement>
	/// <requirement>F:checkout_shipping</requirement>
	/// <requirement>C:checkout_shipping_cont</requirement>
	/// <requirement>F:checkout_payment</requirement>
	/// <requirement>F:checkout_payment_method</requirement>
	/// <requirement>F:checkout_summarize</requirement>
	/// <requirement>C:checkout_payment_method</requirement>
	public class Checkout : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.HyperLink HyperLink1;
		protected System.Web.UI.WebControls.Label Email;
		protected System.Web.UI.WebControls.TextBox Name;
		protected System.Web.UI.WebControls.TextBox Address1;
		protected System.Web.UI.WebControls.TextBox Address2;
		protected System.Web.UI.WebControls.TextBox City;
		protected System.Web.UI.WebControls.DropDownList State;
		protected System.Web.UI.WebControls.TextBox Zip;
		protected System.Web.UI.WebControls.RadioButton LastPayment;
		protected System.Web.UI.WebControls.RadioButton NewPayment;
		protected System.Web.UI.WebControls.DropDownList CCType;
		protected System.Web.UI.WebControls.TextBox CCNumber;
		protected System.Web.UI.WebControls.TextBox CCExpMonth;
		protected System.Web.UI.WebControls.TextBox CCExpYear;
		protected System.Web.UI.WebControls.Label Order;
		protected System.Web.UI.WebControls.Label Tax;
		protected System.Web.UI.WebControls.Label Shipping;
		protected System.Web.UI.WebControls.Button Button;
		protected System.Web.UI.WebControls.Label Total;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			if (!this.IsPostBack)
			{
				// get user id (email) and retrieve customer record
				FormsIdentity user = (FormsIdentity) User.Identity;
				FormsAuthenticationTicket auth = user.Ticket;
				CustomerDB db = (CustomerDB) Application.Get("CustomerDB");
				DbDataAdapter dataAdapter = db.CreateDataAdapter(" WHERE Email = '" + auth.Name + "'");

				DataTable customer = new DataTable();
				dataAdapter.Fill(customer);

				// set fields with values loaded from database
				Email.Text = (string) customer.Rows[0]["Email"];
				Name.Text = (string) customer.Rows[0]["Name"];
				Address1.Text = (string) customer.Rows[0]["Address1"];
				Address2.Text = (string) customer.Rows[0]["Address2"];
				City.Text = (string) customer.Rows[0]["City"];

				// create list of states
				ArrayList states = new ArrayList();
				states.Add("AZ");
				states.Add("NJ");
				states.Add("NY");
				states.Add("TX");
				State.DataSource = states;
				State.DataBind();

				// select state from database
				for (int i = 0; i < states.Count; i++)
					if (((string)states[i]) == (string) customer.Rows[0]["State"])
						State.SelectedIndex = i;

				Zip.Text = (string) customer.Rows[0]["Zip"];

				// create credit card types and select last used one
				ArrayList cctypes = new ArrayList();
				cctypes.Add("Visa");
				cctypes.Add("Master");
				CCType.DataSource = cctypes;
				CCType.DataBind();
				if ((string) customer.Rows[0]["CCType"] == "Master")
					CCType.SelectedIndex = 1;
				else
					CCType.SelectedIndex = 0;

				CCNumber.Text = (string) customer.Rows[0]["CCNumber"];
				CCExpMonth.Text = (string) customer.Rows[0]["CCExpMonth"];
				CCExpYear.Text = (string) customer.Rows[0]["CCExpYear"];

				if (CCNumber.Text.Length == 0)
					NewPayment.Checked = true;
				else
					LastPayment.Checked = true;

				// update the order summary
				this.UpdateSummary();
			}

			// do not cache this page
			Response.Expires = -1;
		}

		/// <summary>
		/// Computes order total, tax and shipping and updates labels on web page.
		/// </summary>
		public void UpdateSummary()
		{
			// retrieve shopping cart information from session object
			ShoppingCart cart = (ShoppingCart) Session["ShoppingCart"];
			if (cart == null)
			{
				cart = new ShoppingCart(Session.SessionID);
				Session["ShoppingCart"] = cart;
			}

			if (cart.Count == 0)
			{
				Response.Redirect(@"..\Empty.aspx");
			}

			// update order label
			double total = cart.GetTotalOrder(Server.MapPath(@"..\products.xml"));
			Order.Text = total.ToString("C");

			// compute tax
			double tax = TaxCalculator.Compute(total, State.SelectedValue);
			Tax.Text = tax.ToString("C");

			// compute shipping
			double weight = cart.GetTotalWeight(Server.MapPath(@"..\products.xml"));
			double shipping = ShippingCostCalculator.Compute(0, weight, State.SelectedValue);
			Shipping.Text = shipping.ToString("C");

			// update total
			total += shipping + tax;
			Total.Text = total.ToString("C");
		}

		/// <summary>
		/// Called when the state changed. Updates the order summary.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void State_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			// Update summary. Shipping or tax might change.
			this.UpdateSummary();
		}

		/// <summary>
		/// Creates a new order based on the items in the shopping cart and the given customer record.
		/// After the order was saved the items in the shopping cart are removed.
		/// </summary>
		/// <param name="customer">Customer record.</param>
		private void SaveOrder(DataTable customer)
		{
			ShoppingCart cart = (ShoppingCart) Session["ShoppingCart"];
			if (cart != null)
			{
				// Get path for a temporary file
				string file = (string) ConfigurationSettings.AppSettings["OrderStore"] + "\\order_" + Session.SessionID.ToString() + ".xml";

				// create a temp XML file
				XmlTextWriter writer = new XmlTextWriter(File.CreateText(file));
				writer.Formatting = Formatting.Indented;

				writer.WriteStartDocument();
				writer.WriteStartElement("doc");

				writer.WriteStartElement("customer");

				writer.WriteElementString("Email", (string) customer.Rows[0]["Email"]);
				writer.WriteElementString("Name", (string) customer.Rows[0]["Name"]);
				writer.WriteElementString("Address1", (string) customer.Rows[0]["Address1"]);
				writer.WriteElementString("Address2", (string) customer.Rows[0]["Address2"]);
				writer.WriteElementString("City", (string) customer.Rows[0]["City"]);
				writer.WriteElementString("State", (string) customer.Rows[0]["State"]);
				writer.WriteElementString("Zip", (string) customer.Rows[0]["Zip"]);
				writer.WriteElementString("CCType", (string) customer.Rows[0]["CCType"]);
				writer.WriteElementString("CCNumber", (string) customer.Rows[0]["CCNumber"]);
				writer.WriteElementString("CCExpMonth", (string) customer.Rows[0]["CCExpMonth"]);
				writer.WriteElementString("CCExpYear", (string) customer.Rows[0]["CCExpYear"]);

				writer.WriteStartElement("Charged");
				writer.WriteAttributeString("Order", Order.Text);
				writer.WriteAttributeString("Tax", Tax.Text);
				writer.WriteAttributeString("Shipping", Shipping.Text);
				writer.WriteAttributeString("Total", Total.Text);
				writer.WriteEndElement();	// charged
				writer.WriteEndElement();	// customer

				writer.WriteStartElement("order");

				IDictionaryEnumerator i = cart.GetEnumerator();
				while(i.MoveNext())
				{
					ShoppingItem item = (ShoppingItem) i.Value;
					writer.WriteStartElement("item");
					writer.WriteAttributeString("product", item.Product);
					writer.WriteAttributeString("option", item.Option);
					writer.WriteAttributeString("quantity", item.Quantity.ToString());
					writer.WriteAttributeString("image", item.ServerPath);
					writer.WriteEndElement();	// item
				}
				writer.WriteEndElement();	// order
				
				writer.WriteEndElement();	// doc
				writer.WriteEndDocument();
				writer.Close();

				// delete all shopping cart items
				cart.Clear();
			}
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Button.Click += new System.EventHandler(this.Button_Click);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		/// <summary>
		/// Called when the place order button is pressed. Updates the customer record and
		/// saves the order.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Button_Click(object sender, System.EventArgs e)
		{
			// get user id (email) and retrieve customer record
			FormsIdentity user = (FormsIdentity) User.Identity;
			FormsAuthenticationTicket auth = user.Ticket;
			CustomerDB db = (CustomerDB) Application.Get("CustomerDB");
			DbDataAdapter dataAdapter = db.CreateDataAdapter(" WHERE Email = '" + auth.Name + "'");

			DataTable customer = new DataTable();
			dataAdapter.Fill(customer);

			// update fields
			customer.Rows[0]["Name"] = Name.Text;
			customer.Rows[0]["Address1"] = Address1.Text;
			customer.Rows[0]["Address2"] = Address2.Text;
			customer.Rows[0]["City"] = City.Text;
			customer.Rows[0]["State"] = State.SelectedValue;
			customer.Rows[0]["Zip"] = Zip.Text;

			// update payment if necessary
			if (NewPayment.Checked)
			{
				customer.Rows[0]["CCType"] = CCType.SelectedValue;
				customer.Rows[0]["CCNumber"] = CCNumber.Text;
				customer.Rows[0]["CCExpMonth"] = CCExpMonth.Text;
				customer.Rows[0]["CCExpYear"] = CCExpYear.Text;
			}
			
			// store back into data base
			dataAdapter.Update(customer);

			// now save the order
			this.SaveOrder(customer);

			// say thank you
			Response.Redirect("Completed.aspx");
		}
	}
}
